@@ -1,27 +1,27 @@
 # pylint: disable=missing-function-docstring
 # pylint: disable=missing-class-docstring
 # pylint: disable=line-too-long
-# '''
-#     A module of utility methods used for generating random data or performing various
-#     actions randomly.
+'''
+    A module of utility methods used for generating random data.
 
-#     ----------
+    ----------
 
-#     Meta
-#     ----------
-#     `author`: Colemen Atwood
-#     `created`: 06-03-2022 10:22:15
-#     `memberOf`: rand
-#     `version`: 1.0
-#     `method_name`: rand
-# '''
+    Meta
+    ----------
+    `author`: Colemen Atwood
+    `created`: 06-03-2022 10:22:15
+    `memberOf`: random_utils
+'''
 
-import random
-import hashlib
-import time
-import string
-from typing import Union
-from faker import Faker
+import random as _random
+import hashlib as _hashlib
+import time as _time
+import string as _string
+from typing import Union as _Union
+from faker import Faker as _Faker
+import utils.random_utils.rand_utils as _ru
+import utils.dict_utils as _obj
+import utils.string_utils as _csu
 # import facades.rand_utils_facade as rand
 
 
@@ -29,16 +29,14 @@ from faker import Faker
 # from utils.object_utils import rand_option as option
 # from utils.string_generation import text,phone,email,url,abstract_name,rand
 
-from utils.dict_utils.dict_utils import get_kwarg as _get_kwarg
+# from utils.dict_utils.dict_utils import get_kwarg as _obj.get_kwarg
 
 
 
-FAKER_INSTANCE:Faker = None
 
-def fake()->Faker:
-    if FAKER_INSTANCE is None:
-        FAKER_INSTANCE = Faker()
-    return FAKER_INSTANCE
+
+def faker()->_Faker:
+    return _Faker()
 
 
 def gen_variations(value):
@@ -56,16 +54,16 @@ def gen_variations(value):
 
 
 def boolean(bias=50):
-    return random.randint(1,100) <= bias
+    return _random.randint(1,100) <= bias
 
 def null_boolean():
     return{
         0: None,
         1: True,
         -1: False,
-    }[random.randint(-1, 1)]
+    }[_random.randint(-1, 1)]
 
-def md5(raw_output: bool = False) -> Union[bytes, str]:
+def md5(raw_output: bool = False) -> _Union[bytes, str]:
     """Generate a random MD5 hash.
 
     If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the MD5 hash
@@ -74,12 +72,12 @@ def md5(raw_output: bool = False) -> Union[bytes, str]:
     :sample: raw_output=False
     :sample: raw_output=True
     """
-    res = hashlib.md5(str(random.random()).encode())
+    res = _hashlib.md5(str(_random.random()).encode())
     if raw_output:
         return res.digest()
     return res.hexdigest()
 
-def sha1(raw_output: bool = False) -> Union[bytes, str]:
+def sha1(raw_output: bool = False) -> _Union[bytes, str]:
     """Generate a random SHA1 hash.
 
     If ``raw_output`` is ``False`` (default), a hexadecimal string representation of the SHA1 hash
@@ -88,32 +86,32 @@ def sha1(raw_output: bool = False) -> Union[bytes, str]:
     :sample: raw_output=False
     :sample: raw_output=True
     """
-    res = hashlib.sha1(str(random.random()).encode())
+    res = _hashlib.sha1(str(_random.random()).encode())
     if raw_output:
         return res.digest()
     return res.hexdigest()
 
-def sha256(raw_output:bool=False) -> Union[bytes, str]:
-    res = hashlib.sha256(str(random.random()).encode())
+def sha256(raw_output:bool=False) -> _Union[bytes, str]:
+    res = _hashlib.sha256(str(_random.random()).encode())
     if raw_output:
         return res.digest()
     return res.hexdigest()
 
-def past_date(days:Union[int,None]=None)->int:
+def past_date(days:_Union[int,None]=None)->int:
     if days is None:
-        days = random.randint(1,800)
-    seconds = random.randint(1,86400)
-    return int(time.time()) - ((days * 86400) + seconds)
+        days = _random.randint(1,800)
+    seconds = _random.randint(1,86400)
+    return int(_time.time()) - ((days * 86400) + seconds)
 
-def future_date(days:Union[int,None]=None)->int:
+def future_date(days:_Union[int,None]=None)->int:
     if days is None:
-        days = random.randint(1,800)
-    seconds = random.randint(1,86400)
-    return int(time.time()) + ((days * 86400) + seconds)
+        days = _random.randint(1,800)
+    seconds = _random.randint(1,86400)
+    return int(_time.time()) + ((days * 86400) + seconds)
 
 def rand(length=12, **kwargs):
     '''
-        Generates a cryptographically secure random string.
+        Generates a cryptographically secure random _string.
 
 
         ----------
@@ -141,7 +139,7 @@ def rand(length=12, **kwargs):
             !"#$%&'()*+,-./:;<=>?@[]^_`{|}~
 
         `exclude`=[] {string|list}
-            Characters to exclude from the random string.
+            Characters to exclude from the random _string.
 
         Return
         ----------
@@ -149,34 +147,34 @@ def rand(length=12, **kwargs):
             A random string of N length.
     '''
 
-    uppercase = _get_kwarg(['upper case', 'upper'], True, bool, **kwargs)
-    lowercase = _get_kwarg(['lower case', 'lower'], True, bool, **kwargs)
-    digits = _get_kwarg(['digits', 'numbers', 'numeric', 'number'], True, bool, **kwargs)
-    symbols = _get_kwarg(['symbols', 'punctuation'], False, bool, **kwargs)
-    exclude = _get_kwarg(['exclude'], [], (list, string), **kwargs)
+    uppercase = _obj.get_kwarg(['upper case', 'upper'], True, bool, **kwargs)
+    lowercase = _obj.get_kwarg(['lower case', 'lower'], True, bool, **kwargs)
+    digits = _obj.get_kwarg(['digits', 'numbers', 'numeric', 'number'], True, bool, **kwargs)
+    symbols = _obj.get_kwarg(['symbols', 'punctuation'], False, bool, **kwargs)
+    exclude = _obj.get_kwarg(['exclude'], [], (list, str), **kwargs)
 
     choices = ''
     if uppercase is True:
-        choices += string.ascii_uppercase
+        choices += _string.ascii_uppercase
     if lowercase is True:
-        choices += string.ascii_lowercase
+        choices += _string.ascii_lowercase
     if digits is True:
-        choices += string.digits
+        choices += _string.digits
     if symbols is True:
-        choices += string.punctuation
+        choices += _string.punctuation
 
     if len(exclude) > 0:
         if isinstance(exclude, str):
             exclude = list(exclude)
-        for e in exclude:
-            choices = choices.replace(e, '')
+        for exd in exclude:
+            choices = choices.replace(exd, '')
 
-    return ''.join(random.SystemRandom().choice(choices) for _ in range(length))
+    return ''.join(_random.SystemRandom().choice(choices) for _ in range(length))
 
 def text(minimum=10,maximum=500,null_bias=0):
     '''
-        Wrapper method for fake().text()
-        This adds the ability to randomly return null instead of the string.
+        Wrapper method for faker().text()
+        This adds the ability to randomly return null instead of the _string.
 
         ----------
 
@@ -208,10 +206,10 @@ def text(minimum=10,maximum=500,null_bias=0):
         null_bias = 50 if null_bias is True else 0
 
     if null_bias:
-        if fake().boolean(null_bias):
+        if faker().boolean(null_bias):
             return None
 
-    val = fake().text()[:random.randint(minimum,maximum)]
+    val = faker().text()[:_random.randint(minimum,maximum)]
     val = val.replace("'","")
     return val
 
@@ -243,8 +241,8 @@ def phone(bias=50):
         * @xxx [06-03-2022 07:25:42]: documentation for phone
     '''
 
-    if fake().boolean(bias):
-        return fake().phone_number()
+    if faker().boolean(bias):
+        return faker().phone_number()
     return None
 
 def email(bias=50):
@@ -275,8 +273,8 @@ def email(bias=50):
         `method_name`: email
         * @xxx [06-03-2022 07:25:42]: documentation for email
     '''
-    if fake().boolean(bias):
-        return fake().free_email()
+    if faker().boolean(bias):
+        return faker().free_email()
     return None
 
 def url(bias=50):
@@ -307,8 +305,8 @@ def url(bias=50):
         `method_name`: url
         * @xxx [06-03-2022 07:25:42]: documentation for url
     '''
-    if fake().boolean(bias):
-        return fake().url()
+    if faker().boolean(bias):
+        return faker().url()
     return None
 
 def abstract_name(**kwargs):
@@ -340,8 +338,8 @@ def abstract_name(**kwargs):
         `method_name`: abstract_name
         * @xxx [06-03-2022 07:38:26]: documentation for abstract_name
     '''
-    bias = _get_kwarg(['bias'], 100, (int), **kwargs)
-    if fake().boolean(bias):
+    bias = _obj.get_kwarg(['bias'], 100, (int), **kwargs)
+    if faker().boolean(bias):
         return f"{_rand_adjective()} {_rand_noun()}".title()
     return None
 
@@ -768,6 +766,105 @@ def _rand_noun():
 def _rand_option(options):
     list_len = len(options)
     try:
-        return options[random.randint(0, list_len)]
+        return options[_random.randint(0, list_len)]
     except IndexError:
         return _rand_option(options)
+
+
+
+def gender()->str:
+    '''
+        Randomly generate a gender "male" or "female"
+
+        ----------
+
+
+        Return {str}
+        ----------------------
+        The gender.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-04-2022 10:11:34
+        `memberOf`: rand_generation
+        `version`: 1.0
+        `method_name`: gender
+        * @xxx [06-04-2022 10:12:06]: documentation for gender
+    '''
+
+
+    return _ru.option(["male","female"])
+
+def user(**kwargs)->dict:
+    '''
+        Randomly generate a user data dictionary.
+
+        ----------
+
+        Keyword Arguments
+        -------------------------
+        [`gender`=None] {str|None}
+            The gender of the user (male,female), if not provided one is randomly selected.
+
+        [`password`=None] {str|None}
+            The password for the user, if not provided one is randomly generated.
+
+        Return {dict}
+        ----------------------
+        A user dictionary:
+        {
+            `gender`:"female",
+            `first_name`:"Sarah",
+            `last_name`:"Paulson",
+            `password`:"Zxbp43JMGuPm",
+            `email`:"dork@gmail.com",
+            `phone`:"806.355.2586",
+            `birthday`:123456789,
+        }
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-04-2022 10:12:17
+        `memberOf`: rand_generation
+        `version`: 0.0.0
+        `method_name`: user
+        * @xxx [06-04-2022]: documentation for user
+    '''
+
+
+    fake = faker()
+
+    data = {
+        "gender":"",
+        "first_name":"",
+        "last_name":"",
+        "password":_obj.get_kwarg(['password','pass'], rand(), (None,str), **kwargs),
+        "email":"",
+        "phone":"",
+        "birthday":past_date(),
+    }
+
+    ugen = _obj.get_kwarg(['gender'], None, (None,str), **kwargs)
+    # data['password'] = _obj.get_kwarg(['password','pass'], rand(), (None,str), **kwargs)
+
+    if ugen is None:
+        data['gender'] = gender()
+
+    if isinstance(ugen,(str)):
+        gen = _csu.determine_gender(ugen)
+        if gen is None:
+            data['gender'] = gender()
+        else:
+            data['gender'] = gen
+
+    if data['gender'] == "female":
+        data['first_name'] = fake.first_name_female()
+    if data['gender'] == "male":
+        data['first_name'] = fake.first_name_male()
+    data['last_name'] = fake.last_name()
+    data['email'] = fake.free_email()
+    data['phone'] = fake.phone_number()
+
+    return data
\ No newline at end of file
